for что in ["элемент", "множество"]:
    for где in ["элемент", "множество"]:
        for регитр in ["важен", "не важен"]:
            for результат in ["содержит", "не содержит"]:
                print(f"{что} - {где}, регистр {регитр}, результат - {результат}")


# принцип программирования TDD - Test Driven Development
# переводится как разработка, основанная на тестировании
# данный принцип предполагает, что сначала создаются тесты для нужной функции, которые будут проверять все возможные варианты ввода и вывода и только потом пишется сама функция
#  при каждом обновлении кода функции необходимо запустить все тесты и удостоверится, что все они пройдены и нет ошибок

# тесты всегда должны писаться "явным" образом - все входные данные и ожидаемые результаты работы функции должны быть написаны "от руки" - они не должны вычисляться самим тестом
def максимальное_число(список):
    if type(список) != list and type(список) != tuple:
        raise TypeError(f"Эта функция может принять только список и кортеж, был передан {type(список)}")
    # список = [5, 9, 2, 3, 15]
    наибольшее = список[0]
    for i in range(len(список)):
    # for элемент in список
        элемент = список[i]
        if элемент > наибольшее:
            наибольшее=элемент
    return наибольшее 

def максимальное_число_сортировка(список):
    список.sort()
    return список[-1]

def тест():
    название_теста = "список чисел"
    данные = [5, 9, 2, 3, 15]
    ожидаемый_ответ = 15
    результат = максимальное_число(данные)
    if результат == ожидаемый_ответ:
        print(f"тест '{название_теста}' пройден")
    else:
        print(f"тест '{название_теста}' не пройден")
        print(f"ожидался ответ   - {ожидаемый_ответ}")
        print(f"получили результат {результат}")

    название_теста = "список чисел с отрицательными"
    данные = [5, 9, 2, 3, 15, -9, -7, 20]
    ожидаемый_ответ = 20
    результат = максимальное_число(данные)
    if результат == ожидаемый_ответ:
        print(f"тест '{название_теста}' пройден")
    else:
        print(f"тест '{название_теста}' не пройден")
        print(f"ожидался ответ   - {ожидаемый_ответ}")
        print(f"получили результат {результат}")

    название_теста = "передан не список"
    данные = "текст"
    ожидаемый_ответ = "ошибка"
    try:
        результат = максимальное_число(данные)
    except:
        результат = "ошибка"
    if результат == ожидаемый_ответ:
        print(f"тест '{название_теста}' пройден")
    else:
        print(f"тест '{название_теста}' не пройден")
        print(f"ожидался ответ   - {ожидаемый_ответ}")
        print(f"получили результат {результат}")
тест()

# данные = ["слово", 5, 10, 15.5]


def печать(
        текст, 
        красный=False,
        зеленый=False, 
        синий=False, 
        желтый=False, 
        стиль=None, 
        не_печатать = False
):

    def определить_стиль(стиль):
        if стиль==None:
            return ""    
        
        СТИЛИ  = {
            "ж":1, 
            "к":3, 
            "ч":4, 
            "з":9 
        }
        накопительная = ""
        for буква in стиль:
            код_стиля = СТИЛИ.get(буква)
            этот_стиль = f";{код_стиля}"
            накопительная+=этот_стиль
        return накопительная
        # при комбинировании жирного и зачеркнутого должеен получиться результат
        # ;1;9
        # курсивый и подчеркнутый - ;3;4
        # жирный и курсивый - ;1;3

    # ОПРЕДЕЛЕННЫЕ СИМВОЛЫ В НАЧАЛЕ И В КОНЦЕ НАПИСАНЫ НА С++ И ОТВЕЧАЮТ ЗА КОДИРОВКУ ЦВЕТА

    # первая часть строки задает стиль цвета и формат шрифта
    # \x1B[ - специальный набор символов, который включает этот режим настройки
    # 38;2; - говорит о том, что след три параметра будут цветом в формате RGB
    # 255;0;0; - цвет в формате RGB
    # 4 - подчеркнутый стиль шрифта
    # m - завершает настройку стиля

    #  последняя часть строки сбрасывает стиль в стандарт, чтобы все дальнейшее писалось обычным цветом
    # результат = "\x1B[38;2;255;0;0;4m" + текст + "\x1B[0m"

    готовый_стиль= определить_стиль(стиль)
    if красный:
        цвет = "255;0;0"
    elif зеленый:
        цвет="0;255;0"
    elif синий:
        цвет="0;0;255"
    elif желтый:
        цвет="200;190;0"
    else:
        цвет="255;255;255"

    результат = f"\x1B[38;2;{цвет}{готовый_стиль}m{текст}\x1B[0m"
    if не_печатать:
        return результат
    print(результат)

# если оставить строки с запуском функции внутри файла с модулем, то они будут выполнены в момент импорта этого модуля
# чтобы этого не допустить, но при этом оставить возможность тестировать этот файл модуля "прямо здесь", например, чтобы провести отладку, надо эти строки убрать внутрь блока "если", который будет проверять откуда запущен этот файл

# ЕСЛИ запуск произошел из этого файла (если он не импортируется): 
if __name__=="__main__":
    печать("123")
    красный = печать("красный", красный=True, не_печатать=True)
    зеленый = печать("зеленый", зеленый=True, не_печатать=True)
    разноцветная_строка = f"{красный} и {зеленый}"
    print(разноцветная_строка)



from .проверка_элементов import содержит
from .терминал import печать

РАЗДЕЛИТЕЛЬ_ПАР="___туеимнокцудеои526гз5ш42670з95826щшозг___"
РАЗДЕЛИТЕЛЬ_КЛЮЧА_ОТ_ЗНАЧЕНИЯ = "___63473673567/*/*/*635/7*396___"
def записать_в_файл(путь, текст):
    with open(путь, "w", encoding="utf-8") as file:
        file.write(текст)
    # r - чтение
    # w - запись
    # a - добавление

def считать_из_файла(путь):
    with open(путь, "r", encoding="utf-8") as file:
        return file.read()

def преобразовать_словарь_в_текст(словарь: dict):
    # если разделитель встречается в одном из значений словаря, то выдать ошибку "В одном из значений словаря содержится ошибка"
    if содержит(
        что=[РАЗДЕЛИТЕЛЬ_КЛЮЧА_ОТ_ЗНАЧЕНИЯ, РАЗДЕЛИТЕЛЬ_ПАР],
        где=list(словарь.values())
    ):
        raise ValueError(печать("В одном из значений словаря найден разделитель",
                         красный=True,
                         не_печатать=True
        ))
    результат = ""
    for ключ, значение in словарь.items():
        пара_в_тексте = f"{ключ}{РАЗДЕЛИТЕЛЬ_КЛЮЧА_ОТ_ЗНАЧЕНИЯ}{значение}"
        результат+=пара_в_тексте + РАЗДЕЛИТЕЛЬ_ПАР
    результат=результат[:-len(РАЗДЕЛИТЕЛЬ_ПАР)]
    return результат

def записать_словари_в_файл(путь: str, список_словарей: list) -> None:
    общий_результат = ""
    for словарь in список_словарей:
        результат = преобразовать_словарь_в_текст(словарь)
        общий_результат+=результат + "\n"
        # "\n"  - спец символ, означающий завершение абзаца
        # всё, что будет написано после него - будет написано с новой строки
    записать_в_файл(путь, общий_результат)

def считать_словари_из_файла(путь: str) -> list:
    текст=считать_из_файла(путь)
    список_словарей = []
    for строка in текст.splitlines():
        пары = строка.split(РАЗДЕЛИТЕЛЬ_ПАР)
        словарь = {}
        for пара in пары:
            ключ, значение = пара.split(РАЗДЕЛИТЕЛЬ_КЛЮЧА_ОТ_ЗНАЧЕНИЯ)
            словарь[ключ] = значение
        список_словарей.append(словарь)
    return список_словарей

if __name__ == "__main__":
    # windows разбивает путь к файлу на разные папки с помощью дроби \
    # питон воспринимает этот символ как специальный и ожидает после него дополнительный специпльный символ
    # поэтому если просто скопировать путь виндоуса в файл питона, будет ошибка
    # чтобы этого не допустить надо:
        # либо проставить везде двойные дроби \\ вместо одинарных \ 
        # либо до кавычки поставить букву "r", но тогда не получиться использовать f-строку
    путь = "C:\\Users\\2024.09 Пайтон\\Desktop\\ученики\\РухлядаПолина\\programm_ucheba\\подключаемый модуль\\инструменты\\файл из питона.txt"
    текст = "текст написан в питоне 21.01.25"
    # записать_в_файл(путь, текст)
    # текст_из_файла = считать_из_файла(путь)
    # print(текст_из_файла == текст)

    список_словарей = [{
        "имя" : "Ни___:___кита",
        "фамилия" : "Тарасов",
        "должность" : "Преподаватель",
        "возраст" : "35"
    }, {
        "имя" : "Игорь",
        "фамилия" : "Чай",
        "должность" : "Дворник",
        "возраст" : "58"
    }]
    записать_словари_в_файл(путь, список_словарей)
    результат = считать_словари_из_файла(путь)
    print(результат == список_словарей)


from setuptools import setup
setup(
    name="polinu_tools", # имя модуля, которое будет использоваться при установке в будущем
    version="0.2", #версия модуля. При каждом обновлении(новой отрпавке) надо версию увеличивать
    description="Мои инструменты", #описание. Можно не использовать
    packages=["polinu_tools"], # список с названием папок, которые нужно залить
    author_email="ruhliadapolina@gmail.com", # почта автора пакета
    zip_safe=False, # пароль
    # install_requires=[], список названий модулей, которые нужны для корректной работы этого модуля
)